import click
import prosecode.tangle
import prosecode.weave
from pygments.formatters import LatexFormatter

@click.group()
def cli():
    pass

@cli.command()
@click.argument('mdfile')
@click.option('--srcdir', default = './',
                help='Where to put the generated source code.')
def tangle(mdfile, srcdir):
    """
    Process the markdown file `mdfile`, extracting all code.
    The code is placed in the directory given in the option `--srcdir`.
    """
    prosecode.tangle.tangle(mdfile, srcdir)
    click.echo('Tangled the code in ' + mdfile + ' to ' + srcdir + '.')

@cli.command()
@click.argument('mdfile')
@click.option('--execute', default = False,
                help='Should the code chunks be executed?')
@click.option('--outfile', default = False, help='The file to save to.')
def weave(mdfile, execute, outfile):
    """
    Process the markdown file `mdfile` into a LaTeX document.

    If `--execute` is set to `True`, then the blocks of code will be executed
    and their output placed in the text.

    The output file is specified in the `--outfile` field.  If omitted, it
    will simple change the extension on the input file from `.md` to `.tex`.
    """
    if outfile == False:
        outfile = mdfile.replace('.md', '.tex')
    prosecode.weave.latexweave(mdfile, execute, outfile)
    click.echo('Wove the code.')

@cli.command()
@click.option('--outfile', default = 'pygment_style_defs.tex',
                help='Where to put the generated source code.')
def styledefs(outfile):
    """
    Generate the LaTeX style definitions to be used in building the code.

    The `--outfile` option tells where to save it.
    """
    with open(outfile, 'w') as styledefsfile:
        styledefsfile.write(LatexFormatter().get_style_defs())
    click.echo('Wrote the latex style definitions to ' + outfile + '.')

@cli.command()
@click.argument('mdfile')
@click.option('--srcdir', default = './',
                help='Where to put the generated source code.')
def cleanup(mdfile, srcdir):
    """
    Delete all of the code generated by the tangling of the markdown file.
    """
    prosecode.tangle.cleanup(mdfile, srcdir)
    click.echo('Finished cleaning ' + mdfile + '.')
